local AAA = AzerothAdventureAlbum;
local L = AAA.L;
local Travel, Titans, TidesofV = AAA.Travel, AAA.Titans, AAA.TidesofV;
local Progress;

local CheckList = {};

local function GetPlayerFaction()
    local playerFaction, _ = UnitFactionGroup("player")
    return  playerFaction
end

playerFaction = GetPlayerFaction()

local mapID_Temp, mapID;
local mapType, mapInfo;
local gossipHasClosed = false;
local gossipOption = 0;

--------------------------------------
-----------Build Notification---------
--------------------------------------

local Toast = CreateFrame("Button", nil, UIParent, "AAAlbumNotificationTemplate");
Toast.soundtrack = nil;
Toast.Label:SetText(L["New Card"]);

local function SetNotificationCard(album, entryID)
    Toast.album = album;
    Toast.entryID = entryID;
    Toast.Title:SetText(album[entryID]["Title"]);
    Toast.Background:SetTexCoord(0.27734375,0.77734375,0.27734375,0.4033203125);
    Toast.Background:SetTexture(album[entryID]["Image"]);
    Toast.animIn:Play();
    Toast:Show();
    Toast.soundtrack = album[entryID]["sound"];
    PlaySound(73280);
end

local function Toast_OnClick(self, button)
	if button == "RightButton" then
		self:Hide();
		return
	else
		local Journal = AAAlbum_MainFrame;
		if (not AAAlbumSwitch:IsShown()) then
			Journal:ShowPanel();
		end
        Journal.JournalBottomPanel.FilterButton.CurrentChoice:SetText(MENU_BFA)
        Journal:UpdateList();
		Journal:GoToCard(self.album, self.entryID);
		if self.albumID == 1 then
			Journal.JournalBottomPanel.FilterButton:Enable();
		end
		Journal.VirtualFrame.JournalDisplay.DisplayBottomLayer:SetTexture(Journal.VirtualFrame.JournalDisplayTopLayer.Display:GetTexture());

		Journal.BackButton:Show();
		Journal.ListScrollFrame:SetFrameStrata("HIGH")
		Journal.VirtualFrame:SetFrameStrata("HIGH")
		Journal.ListScrollFrame2:SetFrameStrata("LOW")
		Journal.VirtualFrame.JournalPinButton:Enable();
		Journal.ListScrollFrame:Show();
		Journal.ListScrollFrame:SetAlpha(1);
		Journal.VirtualFrame:Show();
		UIFrameFadeIn(Journal.VirtualFrame, 0, 1, 1)
		UIFrameFadeIn(Journal.ListScrollFrame, 0, 1, 1)

		self:Hide();

		if (Toast.soundtrack ~= nil) then
			PlaySound(Toast.soundtrack);
		end
	end
end

Toast:SetScript("OnClick", Toast_OnClick);
--------------------------------------


--------------------------------------
-----------Check Achievement----------
--------------------------------------
local EventListener = CreateFrame("Frame");

local function TargetID_OnShow(self)
    self:RegisterEvent("PLAYER_TARGET_CHANGED");
    self:SetScript("OnEvent", function(self, event, ...)
        gossipHasClosed = false;        --if player has spoken to specific NPC but failed to meet the criteria. 
    end);
end

local function TargetID_OnHide(self)
    self:UnregisterAllEvents();
    --print("Check npcID: |cFFFF0000OFF|r")
end

local TargeIDListener = CreateFrame("Frame", nil, EL, "AAAlbumEventListenerTemplate");
TargeIDListener:SetScript("OnShow", TargeIDListener_OnShow);
TargeIDListener:SetScript("OnHide", TargeIDListener_OnHide);


--Gossip
hooksecurefunc(C_GossipInfo, "SelectOption", function(index)
    if TargeIDListener:IsShown() then
        gossipOption = index;
    end
end);

local function GossipListener_OnShow(self)
    TargeIDListener:Show();
    gossipHasClosed = false;
    --print(self:GetName()..": |cFF00FF00ON|r")
    self:RegisterEvent("GOSSIP_SHOW");
    self:RegisterEvent("GOSSIP_CLOSED")
    self:RegisterEvent("ITEM_TEXT_CLOSED")
    self:SetScript("OnEvent", function(self, event, ...)
        if event == "GOSSIP_CLOSED" or event == "ITEM_TEXT_CLOSED" then
            gossipHasClosed = true;
        end
    end);
end

local function GossipListener_OnHide(self)
    self:UnregisterAllEvents();
    --print(self:GetName()..": |cFFFF0000OFF|r")
    gossipHasClosed = false
    TargeIDListener:Hide();
end

local GossipListener = CreateFrame("Frame", nil, EL, "AAAlbumEventListenerTemplate");
GossipListener:SetScript("OnShow", GossipListener_OnShow);
GossipListener:SetScript("OnHide", GossipListener_OnHide);


--------------------------------------
---------Check Target's npcID----------
--------------------------------------
local function CheckTargetID(value)
    if value ~= nil and value ~= 0 then
        local targetGUID = UnitGUID("target")
        if targetGUID ~= nil then
            local type, _, _, _, _, npcID, _ = strsplit("-",targetGUID);
            npcID = tonumber(npcID)
            for i, v in ipairs(value) do
                if v == npcID then
                    return true;
                end
            end
        else
            return false;
        end
    elseif value ~= nil and value == 0  then
        return true;
    else
        return false;
    end
end


--------------------------------------
------------Check Coordinate----------
--------------------------------------
--[[mapInfo :
    0 Cosmic            4 Dungeon
    1 World             5 Mirco
    2 Continent         6 Orphan
    3 Zone
/script mapInfo = C_Map.GetMapInfo(C_Map.GetBestMapForUnit("player"))
/script print(mapInfo.mapType);

local FakeInstanceList = {
    -- Although mapType return "4 - dungeon", the player is not actually in any dungeon 
    -- Put such current mapID here **not the instanceMapID
    629, --The Portrait Room, Dalaran
    20, --Tyr's Tomb
    --936 Azshara's Palace, Well of Eternity
    --939 Courtyard of Lights, Well of Eternity 398
}


local function GetInstanceMapID()
    local _, _, difficultyID, _, _, _, _, instanceMapID, _ = GetInstanceInfo();
    --print(instanceMapID);
    return instanceMapID;
end

function isPlayerInInstance(mapID)
    for i, v in ipairs(FakeInstanceList) do
        if v == mapID then
            return false;
        else
            return true;
        end
    end
end
--]]
--/dump C_Map.GetMapInfo(C_Map.GetBestMapForUnit("player"));
local function AssignMapID()
    local BestMapForPlayer = C_Map.GetBestMapForUnit("player")
    if BestMapForPlayer ~= nil then
        local mapInfo = C_Map.GetMapInfo(BestMapForPlayer);
        mapID = BestMapForPlayer
        mapType = mapInfo.mapType;
    end
end

local function GetPlayerPosition()
    local x, y, positionTable
    positionTable = C_Map.GetPlayerMapPosition(mapID, "player")

    if positionTable ~= nil then
        x, y = positionTable:GetXY();
    else
        x, y = 0, 0
    end

    return x, y
end

local function SaveSpecificProgress(albumID, entryID)
    if (albumID == 1) then
        if (not Travel[entryID]["Status"]) then
            SetNotificationCard(Travel, entryID);
            Progress[entryID] = true;              --turn this off so the progress won't be saved.
            Travel[entryID]["Status"] = true;
        end
    elseif (albumID == 2) then
        if (not Titans[entryID]["Status"]) then
            SetNotificationCard(Titans, entryID);
            Progress.Titans[entryID] = true;
            Titans[entryID]["Status"] = true;
        end
    elseif (albumID == 3) then
        if (not TidesofV[entryID]["Status"]) then
            SetNotificationCard(TidesofV, entryID);
            Progress.TidesofV[entryID] = true;
            TidesofV[entryID]["Status"] = true;
        end
    end

    TargeIDListener:Hide();
end

local function SaveProgress(index)
    local albumID = CheckList[index]["Album"];
    local entryID = CheckList[index]["entryID"];
    SaveSpecificProgress(albumID, entryID);
    AAAlbum_MainFrame:UpdateShelf();
end

local function GetCheckListLength()
    if CheckList ~= nil then
    local i = 1;
    
    while (CheckList[i] and CheckList[i]["x1"] ~= nil) do
        i = i + 1;
    end
    return i-1
    else
        return 0
    end
end

local NoJ = 1;

local function LoadList(mapID, CheckList, Album, AlbumNumber) --Album = the name of the table
    for i = 1, #Album do
        if (Album[i]["mapID"] == mapID and (not Album[i]["Status"]) and Album[i]["x1"] ~= nil) then
            CheckList[NoJ] = {};
            CheckList[NoJ]["Status"] = false;
            CheckList[NoJ]["entryID"] = i;
            CheckList[NoJ]["x1"] = Album[i]["x1"];
            CheckList[NoJ]["x2"] = Album[i]["x2"];
            CheckList[NoJ]["y1"] = Album[i]["y1"];
            CheckList[NoJ]["y2"] = Album[i]["y2"];
            CheckList[NoJ]["Album"] = AlbumNumber;
            CheckList[NoJ]["npcID"] = Album[i]["npcID"];
            CheckList[NoJ]["GossipOption"] = Album[i]["GossipOption"];
            CheckList[NoJ]["Faction"] = Album[i]["Faction"];
            NoJ = NoJ + 1;
        end
    end
end

local function RefreshCheckList(mapID)
    --print("Refreshed")
    GossipListener:Hide();

    wipe(CheckList);
    CheckList = {};
    NoJ = 1;
    LoadList(mapID, CheckList, Travel, 1);
    LoadList(mapID, CheckList, Titans, 2);
    LoadList(mapID, CheckList, TidesofV, 3);
end


local CoordinateListener_totalTime = 0;
local throttle, counter= 1, 0;
local CoordinateListener = CreateFrame("Frame", "CoordinateListener", AAAEventsContainer, EventListenerTemplate)

CoordinateListener:SetScript("OnHide", function(self)
    --print(self:GetName()..": |cFFFF0000OFF|r")
end)
CoordinateListener:SetScript("OnShow", function(self)
    CoordinateListener_totalTime = 0
    --print((self:GetName()..": |cFF00FF00ON|r"))
end)
CoordinateListener:SetScript("OnUpdate", function(self, elapsed)
    counter = counter + elapsed
    if counter < throttle then
        return
    end
    counter = 0

    local x, y = GetPlayerPosition()
    

    if(mapID ~= mapID_Temp) then
        RefreshCheckList(mapID);        --ZONE_CHANGED event fires every time you enter a "subzone". But we only need to refresh the list when entering a new zone.
        mapID_Temp = mapID;
    end

    local CheckListLength = GetCheckListLength();
    if CheckListLength == 0 and mapType~=0 and mapType~=1 and mapType~=2 then
        self:Hide()
    end

    for i = 1, CheckListLength do
        --print(CheckList[i]["entryID"])
        if (not CheckList[i]["Status"] and (x > CheckList[i]["x1"] and x < CheckList[i]["x2"]) and (y > CheckList[i]["y1"]  and y < CheckList[i]["y2"]) ) then
            --print("Right Location")
            AAAlbum_MinimapButton.Bling.animIn:Play();
            if CheckList[i]["npcID"] == nil then
                SaveProgress(i)
                RefreshCheckList(mapID)                             --refresh checklist after earning one
            else
                if CheckTargetID(CheckList[i]["npcID"]) then       --if the criteria include npcID and target npcID = that npcID
                    --print("Right npcID")
                    if CheckList[i]["Faction"] == nil or playerFaction == CheckList[i]["Faction"] then    --the card is not faction-specific, or player match the faction
                        if CheckList[i]["GossipOption"] == nil then                                      
                            SaveProgress(i);
                            RefreshCheckList(mapID)
                        elseif CheckList[i]["GossipOption"] == 0 then
                            GossipListener:Show();
                            if gossipHasClosed then
                                SaveProgress(i);
                                RefreshCheckList(mapID)
                            end
                        else
                            GossipListener:Show();
                            if gossipHasClosed and gossipOption == CheckList[i]["GossipOption"] then
                                SaveProgress(i);
                                RefreshCheckList(mapID)
                            end
                        end
                    else                                                                                  --If the faction doesn't match, just save
                        SaveProgress(i);
                        RefreshCheckList(mapID)                       
                    end
                end
            end

            CheckListLength = GetCheckListLength();
            break;
        end
    end    
end)

--------------------------------------
local ListenerTrigger = CreateFrame("Frame")
ListenerTrigger:RegisterEvent("ZONE_CHANGED");
ListenerTrigger:RegisterEvent("PLAYER_ENTERING_WORLD");
ListenerTrigger:RegisterEvent("ZONE_CHANGED_INDOORS")
ListenerTrigger:RegisterEvent("ZONE_CHANGED_NEW_AREA")
ListenerTrigger:SetScript("OnEvent",function(self, event, ...)
    if event == "PLAYER_ENTERING_WORLD" then
        if not self.hasLoaded then
            self.hasLoaded = true;
            Progress = AzerothAdventureAlbumProgress;
        end
    end
    AssignMapID()
    --print(mapID.."/".. mapType)
    CoordinateListener:Show();
    GossipListener:Hide();
end)



